--*******************************************************************************************************************************
--
-- Description: This module is used to dynamically optimize the FPGA IOB timing at the chip interface so that static timing
--				constraints are not necessary. A state machine continuously adjusts the phase relationship of incoming data
--				to the global clock registering the data so that setup and hold margins are maximized. The resulting solution
--				will set the IDELAY tap values to a position immediately following the clock transition.
--					
--				This core is designed to lock clocks from multiple sources, ensuring that all sources are synchronized
--				to the same edge of global clock.
--					
-- Rev History: 01-10-2018:	Replaced asynchronous reset with synchronous reset.
--				08-04-2016:	Modified the leading and trailing edge filter type to eliminate synthesis warnings.
-- 				08-21-2015:	Corrected a problem with solutions that end on the final tap value.
-- 				07-14-2015:	Changed the operation to continuously correct for deviations in the forwarded clock.
--				07-30-2014:	Added output port to indicate whether search located a rising or falling edge.
--				08-21-2012:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all; 

entity clock_lock is

generic(
	SOURCES : natural;
	MAX_SEARCH : natural;
	RESET_IF_NOSOL : boolean
);

port (
	-- Reset and clocks supplied by the application logic.
	srst         	: in std_logic;		-- Active high synchronous reset.       

	-- Signals used to trigger and monitor the clock lock function.
	lock_start		: in std_logic; 	-- Initiates the clock lock sequence that sets IODELAY values for data inputs. 
	lock_busy		: out std_logic; 	-- Logic one indicates that the clock lock process is active. 
	lock_status		: out std_logic_vector(SOURCES-1 downto 0);	-- Logic one indicates that a solution was found.
		
	-- Connections to the IDELAY function in the IOBs that need to be phase aligned.
	clk_lck_clk		: in std_logic;									-- Reference clock used to optimize IOB timing.
	clk_fwd			: in std_logic_vector(SOURCES-1 downto 0);		-- Forwarded clock signal registered in the IOB.
	clk_edge_search	: in std_logic_vector(1 downto 0);				-- Logic one enables search for the falling edge (bit 0) and/or rising edge (bit 1) of the forwarded clock.
	clk_edge_detect	: out std_logic_vector(SOURCES-1 downto 0);		-- Reports which edge was detected by the search, rising (0) or falling (1).
	fwd_counter		: in std_logic_vector((5*SOURCES)-1 downto 0);	-- Tap count reported by the IDELAY attached to the forwarded clock.
	fwd_ld			: out std_logic_vector(SOURCES-1 downto 0);		-- Logic one loads the starting tap value in the IDELAY attached to the forwarded clock.
	fwd_ce			: out std_logic_vector(SOURCES-1 downto 0);		-- Logic one enables the increment/decrement function in the IDELAY attached to the forwarded clock.
	fwd_inc			: out std_logic_vector(SOURCES-1 downto 0);		-- Logic one increments the tap value and logic zero decrements the tap value in the IDELAY attached to the forwarded clock.
	data_ld			: out std_logic_vector(SOURCES-1 downto 0);		-- Logic one loads the starting tap value in the IDELAY attached to the data bits.
	data_ce			: out std_logic_vector(SOURCES-1 downto 0);		-- Logic one enables the increment/decrement function in the IDELAY attached to the data bits.
	data_inc		: out std_logic_vector(SOURCES-1 downto 0)		-- Logic one increments the tap value and logic zero decrements the tap value in the IDELAY attached to the data bits.
);
end clock_lock;

architecture behavioral of clock_lock is

	type FIR is array (SOURCES-1 downto 0) of unsigned(5 downto 0);		-- Type definition for a small 1-bit FIR filter.
	signal tap				: FIR;										-- Five tap delay line to create a six tap FIR filter.
	type FILTER is array (SOURCES-1 downto 0) of std_logic;				-- Type definition for filter calculation results.
	signal filter_lead		: FILTER;									-- Leading edge filter calculation result.
	signal filter_trail		: FILTER;									-- Trailing edge filter calculation result.
		
	type CLK_LCK_STATE is (idle, load, init, search, monitor, adjust);	-- Type definitions for a state machine to command the IDELAY.
	signal idly_state 		: CLK_LCK_STATE;							-- Current state.
	signal timer			: unsigned(3 downto 0);						-- General purpose timer.
	signal toggle			: std_logic;								-- Toggle before (0) and after (1) clock transition to monitor changes in position.
	signal lock_error		: unsigned(SOURCES-1 downto 0);				-- Logic one indicates that the current search was unsuccessful.
	signal clk_edge_local	: std_logic_vector(SOURCES-1 downto 0);		-- Local replica of the clk_edge_detect output port.
	signal fwd_ce_local		: unsigned(SOURCES-1 downto 0);				-- Local replica of the fwd_ce output port.
	signal start			: std_logic;								-- Lock start input reclocked in the clk_lck_clk domain.
	signal fwd_counter_spd	: std_logic_vector((5*SOURCES)-1 downto 0);	-- Registered version of input fwd_counter to improve timing performance.
	
begin                    

--************************************************* STATE MACHINE TRANSITIONS **************************************************

	process(clk_lck_clk)
	begin
		if rising_edge(clk_lck_clk) then
					
			if srst = '1' then
			
				idly_state <= idle;
				start <= '0';
				
			else
				
				-- Reclock the lock start input to the local clock domain.
				start <= lock_start;
		
				-- The following state machine searches for the IODELAY setting that maximizes timing margin.
				case idly_state is		
				
					-- Remain idle while waiting for the application logic to initiate a clock lock request.
					when idle =>
						-- Detect the lock_start pulse to begin the clock lock process.	
						if start = '1' then
							idly_state <= load;
						end if;
						
					-- Load the starting delay values and pause to allow the output to settle.
					when load =>
						-- Pause for one complete cyle of the timer so that it is zero when entering the next state.
						if timer = x"F" then
							idly_state <= init;
						end if;
						
					-- Initialize the FIR filter that is used to detect a clock edge.
					when init =>
						-- Increment six taps to fill the FIR filter pipeline.
						-- Note: It takes six clocks to produce the first valid results in the calculation registers.
						if timer = x"6" then
							idly_state <= search;
						end if;
						
					-- Find the requested edge of the forwarded clock(s).
					when search =>
						-- The search is complete when all of the clock edges are detected.
						if to_integer(fwd_ce_local) = 0 then
							if to_integer(lock_error) = 0 then
								-- Begin the monitoring function if the search detected an edge.
								idly_state <= monitor;
							elsif RESET_IF_NOSOL = true then
								-- Discontinue the search and return to idle.
								idly_state <= idle;
							else
								-- Start over if the search did not detect an edge.
								idly_state <= load;
							end if;
						end if;
						
					-- Monitor for changes in the clock edge position.
					-- Note:  This process will remove offsets produced by the FIR filter.
					when monitor =>
						idly_state <= adjust;
						
					-- Make any adjustment needed to the delay values based on the current edge position.
					when adjust =>
						-- Pause for one complete cyle of the timer to fill the FIR filter taps with stable values.
						if timer = x"F" then
							idly_state <= monitor;
						end if;
						
					-- Define the default state to be null.
					when others =>
						null;
						
				end case;
			
			end if;
		end if;
	end process;
	
--*************************************************** STATE MACHINE EVENTS *****************************************************
			
	-- Assign the clock forward bit to the first filter tap.
	FWD_TAP : for i in 0 to SOURCES-1 generate
		tap(i)(0) <= clk_fwd(i);
	end generate;
	
	process(clk_lck_clk)
	begin
		if rising_edge(clk_lck_clk) then

			if srst = '1' then
				for i in 0 to SOURCES-1 loop
					tap(i)(5 downto 1) <= (others => '0');
					filter_lead(i) <= '0';
					filter_trail(i) <= '0';
				end loop;
				lock_error <= (others => '0');
				toggle <= '0';
				
			else
				
				-- Register the clock forward tap counter to improve timing performance.
				fwd_counter_spd <= fwd_counter;
				
				-- Create a five tap delay line for each source to use as a filter on the forwarded clock signal.
				for i in 0 to SOURCES-1 loop
					for j in 0 to 4 loop
						tap(i)(j+1) <= tap(i)(j);
					end loop;
				end loop;
				
				-- Filter the leading and trailing edge of the clock transition on each input to eliminate false triggers.
				-- Note: It takes six clocks to produce the first valid results in the calculation registers.
				for i in 0 to SOURCES-1 loop
					-- The leading edge filter uses a simple majority (two out of three) decision.
					if to_integer(tap(i)(2 downto 0)) = 3 or to_integer(tap(i)(2 downto 0)) > 4 then
						filter_lead(i) <= '1';
					else
						filter_lead(i) <= '0';
					end if;
					-- The trailing edge filter uses a simple majority (two out of three) decision.
					if to_integer(tap(i)(5 downto 3)) = 3 or to_integer(tap(i)(5 downto 3)) > 4 then
						filter_trail(i) <= '1';
					else
						filter_trail(i) <= '0';
					end if;
				end loop;
				
				-- Create a counter that can be used as a timer.
				if idly_state = init or idly_state = load or idly_state = adjust then
					timer <= timer + x"1";
				else
					timer <= (others => '0');
				end if;
						
				-- Load each IDELAY to the starting tap value.
				for i in 0 to SOURCES-1 loop
					if idly_state = load then
						fwd_ld(i) <= '1';
						data_ld(i) <= '1';
					else
						fwd_ld(i) <= '0';
						data_ld(i) <= '0';
					end if;
				end loop;
						
				-- Set the IDELAY control bit to increment or decrement tap values.
				if idly_state = monitor then
					fwd_inc <= (others => '0');
				elsif idly_state = adjust then
					fwd_inc <= (others => '1');
					data_inc <= (others => toggle);
				else
					fwd_inc <= (others => '1');
					data_inc <= (others => '1');
				end if;
						
				-- Set the IDELAY enable bit that allows tap values to increment or decrement.
				if idly_state = init then
					-- Enable all IDELAY tap values to increment during the FIR filter initialization.
					fwd_ce_local <= (others => '1');
					data_ce <= (others => '1');
					lock_error <= (others => '0');
				elsif idly_state = search then
					-- Search for edges on any clock that hasn't already been detected.
					for i in 0 to SOURCES-1 loop
						-- Stop searching when a solution is found because other clocks may still be searching.
						if fwd_ce_local(i) = '1' then
							-- Detect the falling edge of the forwarded clocks while ignoring glitches one clock in duration.
							-- Note: The falling edge is a low to high transition because the forwarded clock is progressively delayed.
							if clk_edge_search(1) = '1' and filter_trail(i) = '0' and filter_lead(i) = '1' then
								-- Stop incrementing all IDELAY tap values.
								fwd_ce_local(i) <= '0';
								data_ce(i) <= '0';
								-- Set the clock edge status bit to indicate that a rising edge was detected.
								clk_edge_local(i) <= '1';
							-- Detect the rising edge of the forwarded clocks while ignoring glitches one clock in duration.
							-- Note: The rising edge is a high to low transition because the forwarded clock is progressively delayed.
							elsif clk_edge_search(0) = '1' and filter_trail(i)= '1' and filter_lead(i) = '0' then
								-- Stop incrementing all IDELAY tap values.
								fwd_ce_local(i) <= '0';
								data_ce(i) <= '0';
								-- Set the clock edge status bit to indicate that a falling edge was detected.
								clk_edge_local(i) <= '0';
							-- Stop searching if the end of the window is reached.
							-- Note: The counter is registered so the maximum value is decremented by one.
							elsif fwd_counter_spd((5*i)+4 downto 5*i) = std_logic_vector(to_unsigned(MAX_SEARCH-1,5)) then
								-- Stop incrementing all IDELAY tap values.
								fwd_ce_local(i) <= '0';
								data_ce(i) <= '0';
								-- Set the lock status bit to indicate that the search failed.
								lock_error(i) <= '1';
						end if;
							end if;
					end loop;
				elsif idly_state = monitor and toggle = '0' then
					-- Decrement only the clock forward IDELAY tap value one count to determine whether the clock transition has moved backward.
					fwd_ce_local <= (others => '1');
					data_ce <= (others => '0');
				elsif idly_state = adjust then
					-- Analyze the current solution after the FIR filter taps fill with stable values.
					if timer = x"F" then
						-- Analyze the state of the fowarded clock on either side of the detected edge to determine if it has moved.
						for i in 0 to SOURCES-1 loop
							-- Do not make an adjustment if the lower (31) or upper (MAX_SEARCH) edge of the search window is reached.
							-- Note: The counter is registered, but the registered value is current due to the timer delay.
							if fwd_counter_spd((5*i)+4 downto 5*i) = "11111" or fwd_counter_spd((5*i)+4 downto 5*i) = std_logic_vector(to_unsigned(MAX_SEARCH,5)) then
								-- Restore the clock forward tap value to the original solution.
								fwd_ce_local(i) <= not(toggle);
								data_ce(i) <= '0';
							-- Use the filter taps to determine whether the expected high value is actually low on average.
							elsif clk_edge_local(i) = toggle and filter_lead(i) = '0' and filter_trail(i) = '0' then
								-- Adjust all of the IDELAY tap values to the new solution.
								-- Note: The clock forward IDELAY value is already decremented when determining whether the clock transition has moved backward. 
								fwd_ce_local(i) <= toggle;
								data_ce(i) <= '1';
							-- Use the filter taps to determine whether the expected high low is actually high on average.
							elsif clk_edge_local(i) = not(toggle) and filter_lead(i) = '1' and filter_trail(i) = '1' then
								-- Adjust all of the IDELAY tap values to the new solution.
								-- Note: The clock forward IDELAY value is already decremented when determining whether the clock transition has moved backward. 
								fwd_ce_local(i) <= toggle;
								data_ce(i) <= '1';
							else
								-- Restore the clock forward tap value to the original solution when no change is needed.
								fwd_ce_local(i) <= not(toggle);
								data_ce(i) <= '0';
							end if;
						end loop;
						-- Toggle to the other side of the clock transition.
						toggle <= not(toggle);
					else
						-- Hold the current tap values while the FIR filter taps fill with stable values.
						fwd_ce_local <= (others => '0');
						data_ce <= (others => '0');
					end if;
				else
					-- Hold the current tap values in states that do not make measurements.
					fwd_ce_local <= (others => '0');
					data_ce <= (others => '0');
				end if;

			end if;
		end if;
	end process;
	
--****************************************************** PORT ASSIGNMENTS ******************************************************
			
	fwd_ce <= std_logic_vector(fwd_ce_local);
	clk_edge_detect <= clk_edge_local;

	-- Reclock the following output ports since they will likely drive a different clock domain.	
	process(clk_lck_clk)
	begin
		if rising_edge(clk_lck_clk) then
			if idly_state = idle then
				lock_busy <= '0';
			else
				lock_busy <= '1';
			end if;
			lock_status <= not(std_logic_vector(lock_error));
		end if;
	end process;

end behavioral;
