--*******************************************************************************************************************************
--
-- Description: This module is used to assign source clocks to global buffers and connect them to desired inputs.
--
-- Rev History: 01-10-2018:	Replaced asynchronous reset with synchronous reset.
--				08-04-2016	Added MMCM to generate spread spectrum QDR II+ SRAM clock and removed unused signals.
-- 				06-22-2016	Change type of rx_clk_in and tx_clk_in ports.
-- 				08-20-2014	Removed clk500 from I/O ports.
--				09-04-2013	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;
library UNIMACRO;
use unimacro.Vcomponents.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity clock_route is 
	port (
	-- Reset and clocks.
	sample_clk_in		: in std_logic;						-- IOB sample clock.
    clk200_in			: in std_logic;						-- 200 MHz clock from an external oscillator.
    qdriip_sys_clk		: in std_logic;						-- QDR II+ SRAM system clock.
	
	-- Output reset and clocks.
	rx_clk				: out std_logic;					-- Clock assigned to the receiver datapath.
	tx_clk				: out std_logic;					-- Clock assigned to the transmitter datapath.
    clk200				: out std_logic;					-- 200 MHz clock on a global clock buffer.
    qdriip_clk			: out std_logic						-- Clock assigned to SRAM-A and SRAM-B.
	);	
	end clock_route;

architecture behavioral of clock_route is

	signal clk440fb		: std_logic;					-- 440 MHz spread spectrum MMCM feedback.
	signal clk440out0	: std_logic;					-- 440 MHz spread spectrum MMCM output.
	signal sample_clk	: std_logic;					-- Sample clock on global clock buffer.

begin                    

--**************************************************** GLOBAL CLOCK BUFFERS ****************************************************

	-- Assign the 200 MHz oscillator source to a global clock buffer.
	CLK200_BUFG : BUFG port map (I => clk200_in, O => clk200);
	
	-- Assign the sample clock source to a global clock buffer.
	SAMPLE_CLK_BUFG : BUFG port map (I => sample_clk_in, O => sample_clk);
		
--************************************************* RECEIVER CLOCK ASSIGNMENTS *************************************************

	-- Connect the receiver clock to sample clock if requested.
	RX_CLK_SAMPLE: if RX_CHANNEL_COUNT > 0 and RX_CLK_SELECT = 0 generate
		rx_clk <= sample_clk;
	end generate;
	
	-- Tie the receiver clock low if there are no receiver channels.
	RX_CLK_NONE: if RX_CHANNEL_COUNT = 0 generate
		rx_clk <= '0';
	end generate;
	
--*********************************************** TRANSMITTER CLOCK ASSIGNMENTS ************************************************

	TX_CLK_SAMPLE: if TX_CHANNEL_COUNT > 0 and TX_CLK_SELECT = 0 generate
		-- Connect the transmitter clock to sample clock if requested.
		tx_clk <= sample_clk;
	end generate;
	
	-- Tie the transmitter clock low if there are no transmitter channels.
	TX_CLK_NONE: if TX_CHANNEL_COUNT = 0 generate
		tx_clk <= '0';
	end generate;
	
--************************************************* QDR SRAM CLOCK ASSIGNMENTS *************************************************

	-- Route the 450 MHz QDR II+ system clock directly to the SRAM.
	QDR_CLK_450: if QDR_ADDR_WIDTH > 0 and QDR_SPREAD = false generate
		qdriip_clk <= qdriip_sys_clk;
	end generate;
	
	-- Create a spread spectrum 440 MHz (center frequency) SRAM clock from the 150 MHz QDR II+ system clock.
	-- Note: The spread spectrum clock will continuously vary between 430 MHz and 450 MHz.
	-- Note: The SRAM supports 500 MHz clock rate, but frequencies above 440 MHz may require floorplanning to close timing.
    -- Note: The MMCM automatically locks after power-on.
	QDR_CLK_440: if QDR_ADDR_WIDTH > 0 and QDR_SPREAD = true generate
		-- Instantiate a spread spectrum clock.
		CLK440 : MMCME2_ADV
		generic map (
			BANDWIDTH => "LOW",
			CLKFBOUT_MULT_F => 22.000,
			CLKIN1_PERIOD => 6.667,
			CLKOUT0_DIVIDE_F => 2.5,
			DIVCLK_DIVIDE => 3,
			SS_EN => "TRUE",
			SS_MOD_PERIOD => 4000,
			SS_MODE => "CENTER_HIGH"
		)
		port map (
			CLKOUT0 => clk440out0,
			CLKOUT0B => open,
			CLKOUT1 => open,
			CLKOUT1B => open,
			CLKOUT2 => open,
			CLKOUT2B => open,
			CLKOUT3 => open,
			CLKOUT3B => open,
			CLKOUT4 => open,
			CLKOUT5 => open,
			CLKOUT6 => open,
			DO => open,
			DRDY => open,
			PSDONE => open,
			CLKFBOUT => clk440fb,
			CLKFBOUTB => open,
			CLKFBSTOPPED => open,
			CLKINSTOPPED => open,
			LOCKED => open,
			CLKIN1 => qdriip_sys_clk,
			CLKIN2 => '0',
			CLKINSEL => '1',
			PWRDWN => '0',
			RST => '0',
			DADDR => "0000000",
			DCLK => '0',
			DEN => '0',
			DI => x"0000",
			DWE => '0',
			PSCLK => '0',
			PSEN => '0',
			PSINCDEC => '0',
			CLKFBIN => clk440fb
		);
		-- Map the MMCM divided output to a global clock buffer.
		CLKOUT0_BUFG : BUFG port map (I => clk440out0, O => qdriip_clk);
	end generate;

	-- Assign unused signals if there is no QDR II+ SRAM.
	NO_SRAM : if QDR_ADDR_WIDTH = 0 generate	
		qdriip_clk <= '0';
	end generate;

end behavioral;
