--*******************************************************************************************************************************
--
-- Description: This package declares all of the unique data types used in the SigFPGA reference design.
--					
-- Rev History: 08-04-2016: Added RX_FRAME_BIT, RX_CLOCK_BIT, TX_FRAME_BIT, TX_ENABLE_BIT and TX_CLOCK_BIT types.
-- 				02-18-2016: Changed definition of RX_BUS_IN and OVR_BIT_IN.
-- 				07-14-2015: Added new types to support updated QDR SRAM interface.
-- 				08-11-2014: Added TX_BUS_OUT type.
-- 				09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.model_specific_package.all;

package transceiver_package is

	-- Define the multi-bus data types.
	type DUAL_1 is array (1 downto 0) of std_logic;
	type DUAL_2 is array (1 downto 0) of std_logic_vector(1 downto 0);
	type DUAL_4 is array (1 downto 0) of std_logic_vector(3 downto 0);
	type DUAL_8 is array (1 downto 0) of std_logic_vector(7 downto 0);
	type DUAL_16 is array (1 downto 0) of std_logic_vector(15 downto 0);
	type DUAL_18 is array (1 downto 0) of std_logic_vector(17 downto 0);
	type DUAL_32 is array (1 downto 0) of std_logic_vector(31 downto 0);
	type DUAL_36 is array (1 downto 0) of std_logic_vector(35 downto 0);
	type DUAL_64 is array (1 downto 0) of std_logic_vector(63 downto 0);
	type DUAL_72 is array (1 downto 0) of std_logic_vector(71 downto 0);
	type DUAL_16U is array (1 downto 0) of unsigned(15 downto 0);
	type DUAL_CHU is array (1 downto 0) of unsigned(TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1);
	type DUAL_QDR is array (1 downto 0) of std_logic_vector(zero_to_one(QDR_ADDR_WIDTH)-1 downto 0);
	type AXI_11U is array ((AXI_WIDTH/64)-1 downto 0) of unsigned(10 downto 0);
	
	-- Define the receiver (RX) channel data types.
	type RX_BUS_IN is array (zero_to_one(RX_CHANNEL_COUNT/RX_DATA_SHARE) downto 1) of std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);
	type RX_BUS is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic_vector((RX_ITEM_STUFF*RX_ITEM_SIZE)-1 downto 0);
	type OVR_BIT_IN is array (zero_to_one(RX_CHANNEL_COUNT/RX_DATA_SHARE) downto 1) of std_logic_vector((RX_ITEM_PER_CLK/RX_ITEM_PER_OVR)-1 downto 0);
	type OVR_BIT is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic_vector(RX_ITEM_STUFF-1 downto 0);
	type RX_FRAME_BIT is array (zero_to_one(RX_CHANNEL_COUNT/RX_FRAME_SHARE) downto 1) of std_logic;
	type RX_CLOCK_BIT is array (zero_to_one(ADC_CLK_SOURCES) downto 1) of std_logic;
	type RX_1 is array (zero_to_one(RX_CHANNEL_COUNT) downto 1) of std_logic;

	-- Define the transmitter (TX) channel data types.
	type TX_BUS_OUT is array (zero_to_one(integer(TX_CHANNEL_COUNT/TX_DATA_SHARE))+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic_vector((TX_ITEM_STUFF*TX_ITEM_SIZE)-1 downto 0);
	type TX_BUS is array (zero_to_one(TX_CHANNEL_COUNT)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic_vector((TX_ITEM_STUFF*TX_ITEM_SIZE)-1 downto 0);
	type TX_FRAME_BIT is array (zero_to_one(TX_CHANNEL_COUNT/TX_FRAME_SHARE)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic;
	type TX_ENABLE_BIT is array (zero_to_one(TX_CHANNEL_COUNT/TX_ENABLE_SHARE)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic;
	type TX_CLOCK_BIT is array (zero_to_one(TX_CHANNEL_COUNT/TX_CLOCK_SHARE)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic;
	type TX_1 is array (zero_to_one(TX_CHANNEL_COUNT)+RX_CHANNEL_COUNT downto RX_CHANNEL_COUNT+1) of std_logic;
	
	-- Define the combined transmitter and receiver channel data types.
	type CH_TYPE_1 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic;
	type CH_TYPE_2 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(1 downto 0);
	type CH_TYPE_3 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(2 downto 0);
	type CH_TYPE_4 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(3 downto 0);
	type CH_TYPE_6 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(5 downto 0);
	type CH_TYPE_10 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(9 downto 0);
	type CH_TYPE_13 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(12 downto 0);
	type CH_TYPE_16 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(15 downto 0);
	type CH_TYPE_24 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(23 downto 0);
	type CH_TYPE_32 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(31 downto 0);
	type CH_TYPE_36 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(35 downto 0);
	type CH_TYPE_64 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(63 downto 0);
	type CH_TYPE_128 is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(127 downto 0);
	type CH_TYPE_AXI is array (TX_CHANNEL_COUNT+RX_CHANNEL_COUNT downto 1) of std_logic_vector(AXI_WIDTH-1 downto 0);
	
end transceiver_package;
